@@ -25,8 +25,14 @@ include("../BlockDialog.js");
  * \ingroup ecma_block
  * \brief Renames the active (selected) block.
  */
-function RenameBlock(guiAction) {
+function RenameBlock(guiAction, blockId) {
     Block.call(this, guiAction);
+
+    if (isNull(blockId)) {
+        blockId = RObject.INVALID_ID;
+    }
+
+    this.blockId = blockId;
 }
 
 RenameBlock.prototype = new Block();
@@ -34,7 +40,14 @@ RenameBlock.prototype = new Block();
 RenameBlock.prototype.beginEvent = function() {
     Block.prototype.beginEvent.call(this);
 
-    var blockId = Block.getActiveBlockId();
+    var blockId;
+    if (this.blockId!==RObject.INVALID_ID) {
+        blockId = this.blockId;
+    }
+    else {
+        blockId = Block.getActiveBlockId();
+    }
+
     if (blockId===RObject.INVALID_ID) {
         this.terminate();
         return;
@@ -43,6 +56,13 @@ RenameBlock.prototype.beginEvent = function() {
     var doc = this.getDocument();
 
     var block = doc.queryBlock(blockId);
+    if (isNull(block)) {
+        this.terminate();
+        return;
+    }
+
+    var oldBlockName = block.getName();
+
     var dlg = this.createDialog(doc, block);
     if (isNull(dlg)) {
         this.terminate();
@@ -55,16 +75,24 @@ RenameBlock.prototype.beginEvent = function() {
         return;
     }
 
-    var op = new RAddObjectOperation(newBlock);
-    op.setText(this.getToolTitle());
-    var di = this.getDocumentInterface();
-    di.applyOperation(op);
-    di.clearPreview();
-    di.repaintViews();
+    this.renameBlock(oldBlockName, newBlock, false);
 
     this.terminate();
 };
 
+RenameBlock.prototype.renameBlock = function(oldBlockName, newBlock, useTransactionGroup) {
+    var op = new RAddObjectOperation(newBlock);
+    op.setText(this.getToolTitle());
+    var di = this.getDocumentInterface();
+    var doc = di.getDocument();
+    if (useTransactionGroup) {
+        op.setTransactionGroup(doc.getTransactionGroup());
+    }
+    di.applyOperation(op);
+    di.clearPreview();
+    di.repaintViews();
+};
+
 RenameBlock.prototype.createDialog = function(doc, block) {
-    return new BlockDialog(doc, block);
+    return new BlockDialog(doc, block, true, true);
 };
